@@ -2,9 +2,9 @@
 # Mail::SPF::Mech::PTR
 # SPF record "ptr" mechanism class.
 #
-# (C) 2005-2012 Julian Mehnle <julian@mehnle.net>
+# (C) 2005-2006 Julian Mehnle <julian@mehnle.net>
 #     2005      Shevek <cpan@anarres.org>
-# $Id: PTR.pm 57 2012-01-30 08:15:31Z julian $
+# $Id: PTR.pm 30 2006-11-27 19:55:10Z Julian Mehnle $
 #
 ##############################################################################
 
@@ -23,6 +23,8 @@ use base 'Mail::SPF::SenderIPAddrMech';
 
 use Mail::SPF::Util;
 
+use NetAddr::IP;
+
 use constant TRUE   => (0 == 0);
 use constant FALSE  => not TRUE;
 
@@ -57,11 +59,10 @@ See L<Mail::SPF::Mech/new>.
 
 =back
 
-=item B<new_from_string($text, %options)>: returns I<Mail::SPF::Mech::PTR>;
+=item B<new_from_string($text)>: returns I<Mail::SPF::Mech::PTR>;
 throws I<Mail::SPF::ENothingToParse>, I<Mail::SPF::EInvalidMech>
 
-Creates a new SPF record C<ptr> mechanism object by parsing the string and
-any options given.
+Creates a new SPF record C<ptr> mechanism object by parsing the given string.
 
 =back
 
@@ -143,7 +144,7 @@ sub match {
     $server->count_dns_interactive_term($request);
     return
         Mail::SPF::Util->valid_domain_for_ip_address(
-            $server, $request, $request->ip_address, $self->domain($server, $request))
+            $server, $request->ip_address, $self->domain($server, $request))
         ? TRUE : FALSE;
 }
 
@@ -153,7 +154,7 @@ sub match {
 
 L<Mail::SPF>, L<Mail::SPF::Record>, L<Mail::SPF::Term>, L<Mail::SPF::Mech>
 
-L<http://tools.ietf.org/html/rfc4408>
+L<http://www.ietf.org/rfc/rfc4408.txt>
 
 For availability, support, and license information, see the README file
 included with Mail::SPF.
